"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getExportVersion = exports.importData = exports.exportData = void 0;
var _circle_1 = require("./strokeNodes/_circle");
var _group_1 = require("./strokeNodes/_group");
var _path_1 = require("./strokeNodes/_path");
var _polygon_1 = require("./strokeNodes/_polygon");
var _text_1 = require("./strokeNodes/_text/_text");
/**
 * Export the data of the graph in a certain format that can be used to load the data later. Load using .importData().
 * @returns Data of the graph in a storable and loadable format.
 */
function exportData() {
    this._beforeExport();
    var strokeExport = [];
    this.strokes.forEach(function (stroke) {
        strokeExport.push(stroke.map(function (node) { return node.export(); }));
    });
    return {
        version: 2,
        exportData: strokeExport,
        strokeIndex: this._strokeIndex,
        dimensions: this.dimensions,
        bgType: this.bgType
    };
}
exports.exportData = exportData;
/**
 * Import the data exported by .export() method.
 * @param data Data exported by .export()
 * @returns Self for chaining.
 */
function importData(data) {
    var _this = this;
    var _a;
    this.strokes.forEach(function (stroke) {
        stroke.forEach(function (node) { return node.delete(); });
    });
    this.svg.setAttribute('viewBox', "0 0 " + this.dimensions[0] + " " + this.dimensions[1]);
    this.bgType = (_a = data.bgType) !== null && _a !== void 0 ? _a : this.bgType;
    this.strokes = [];
    if ('version' in data) {
        if (data.version === 2) {
            data.exportData.forEach(function (strokeExport) {
                _this.strokes.push(strokeExport.map(function (strokeNodeData) {
                    var _a, _b, _c, _d, _e;
                    switch (strokeNodeData.type) {
                        case 'circle':
                            var circ = new _circle_1.Circle([0, 0], 0, (_a = strokeNodeData.section) !== null && _a !== void 0 ? _a : 'strokes');
                            circ.import(strokeNodeData);
                            return circ;
                        case 'path':
                            var path = new _path_1.Path('', (_b = strokeNodeData.section) !== null && _b !== void 0 ? _b : 'strokes');
                            path.import(strokeNodeData);
                            return path;
                        case 'text':
                            var text = new _text_1.Text([0, 0], '', (_c = strokeNodeData.section) !== null && _c !== void 0 ? _c : 'strokes');
                            text.import(strokeNodeData);
                            return text;
                        case 'group':
                            var group = new _group_1.GroupNode((_d = strokeNodeData.section) !== null && _d !== void 0 ? _d : 'strokes', []);
                            group.import(strokeNodeData);
                            return group;
                        case 'polygon':
                            var polygon = new _polygon_1.Polygon([], (_e = strokeNodeData.section) !== null && _e !== void 0 ? _e : 'strokes');
                            polygon.import(strokeNodeData);
                            return polygon;
                    }
                }));
            });
        }
    }
    else {
        // legacy deprecated unsafe
        data.exportData.forEach(function (strokeExport) {
            _this.strokes.push(strokeExport.map(function (strokeNodeData) {
                var _a, _b, _c, _d, _e;
                switch (strokeNodeData.type) {
                    case 'circle':
                        var circ = new _circle_1.Circle([0, 0], 0, (_a = strokeNodeData.section) !== null && _a !== void 0 ? _a : 'strokes');
                        circ.importV1(strokeNodeData.data);
                        return circ;
                    case 'path':
                        var path = new _path_1.Path('', (_b = strokeNodeData.section) !== null && _b !== void 0 ? _b : 'strokes');
                        path.importV1(strokeNodeData.data);
                        return path;
                    case 'text':
                        var text = new _text_1.Text([0, 0], '', (_c = strokeNodeData.section) !== null && _c !== void 0 ? _c : 'strokes');
                        text.importV1(strokeNodeData.data);
                        return text;
                    case 'group':
                        var group = new _group_1.GroupNode((_d = strokeNodeData.section) !== null && _d !== void 0 ? _d : 'strokes', []);
                        group.importV1(strokeNodeData.data);
                        return group;
                    case 'polygon':
                        var polygon = new _polygon_1.Polygon([], (_e = strokeNodeData.section) !== null && _e !== void 0 ? _e : 'strokes');
                        polygon.importV1(strokeNodeData.data);
                        return polygon;
                }
            }));
        });
    }
    this._strokeIndex = data.strokeIndex;
    for (var i = 0; i <= this._strokeIndex; i++) {
        this._display(this.strokes[i]);
    }
    this.emit('import', {
        import: data
    });
    return this;
}
exports.importData = importData;
function getExportVersion(data) {
    if ('version' in data)
        return data.version;
    else
        return 1;
}
exports.getExportVersion = getExportVersion;
